# -*- coding: utf-8 -*-
import base64
import datetime
import time
import hashlib

from django.core.mail import EmailMultiAlternatives
from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from django.utils.translation import ugettext as _
from django.conf import settings
from django.template import Context
from django.template.loader import get_template

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from acomea.angel.cms.forms import AngelForm
from acomea.angel.cms.models import AngelSettings
from acomea.angel.models import AttivazioneAngel


class AngelPlugin(CMSPluginBase):
    model = AngelSettings
    name = _("Angel Plugin")
    module = _("Angel")
    admin_preview = False

    def create_angel(self, profilo):
        try:
            attivazione_angel = profilo.attivazione_angel
            return False, attivazione_angel
        except:
            attivazione_angel = AttivazioneAngel()
            attivazione_angel.profilo = profilo
            attivazione_angel.data_attivazione = datetime.date.today()
            attivazione_angel.save()
            return True, attivazione_angel

    def get_angel_status(self, profilo, durata):
        try:
            attivazione_angel = profilo.attivazione_angel
            if attivazione_angel.data_attivazione + datetime.timedelta(days=durata) >= datetime.date.today():
                return 'attivo'
            return 'non_attivo'
        except Exception, e:
            return 'inesistente'

    def render(self, context, instance, placeholder):
        from acomea.angel.forms import EmailForm

        request = context['request']
        user = request.user
        self.render_template = instance.template_name

        app_name = {
            'angel/cms_plugins/angel_costs.html' : 'Angel Costi',
            'angel/cms_plugins/angel_perf.html': 'Angel Performance',
        }[instance.template_name]
        vendor_code = settings.ANGEL_PLUGIN_VENDOR_CODE
        acomea_limited = settings.ANGEL_PLUGIN_LIMITED_CODE
        acomea_full = settings.ANGEL_PLUGIN_FULL_CODE
        timestamp = int(time.time() * 1000)
        costs_url = settings.ANGEL_PLUGIN_COSTS_URL
        perf_url = settings.ANGEL_PLUGIN_PERF_URL

        context.update({
            'object': instance,
            'placeholder': placeholder,
            'costs_url': costs_url,
            'perf_url': perf_url,
            'introduzione': instance.introduzione_app,
            'label_button': instance.label_non_loggato,
            'link_button': instance.link_non_loggato,
            'label_conferma': instance.label_conferma,
            'user_status': 'anonimo',
            'user_profile': 'anonimo',
            'app_name': app_name,
        })

        # return context

        if not instance.status:
            return context

        if user.is_anonymous() or not user.is_active:
            # UTENTE ANONIMO: Non ha la app
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            # UTENTE ANONIMO: Non ha la app
            return context


        # Recupero form di invio mail promemoria
        if user.is_authenticated():
            email_form_class = EmailForm(initial={'email': user.email})
        else:
            email_form_class = EmailForm()

        # Invio email promemoria in caso form non visualizzato
        if request.method == 'POST' and 'email_submit' in request.POST:
            email_form_class = EmailForm(request.POST)
            if email_form_class.is_valid():
                email = request.POST.get('email')
                try:
                    validate_email(email)
                    plaintext = get_template("angel/cms_plugins/email/memo.txt")
                    htmly = get_template("angel/cms_plugins/email/memo.html")
                    d = Context({
                        'username': request.user,
                        'app_name': app_name,
                        'link': request.build_absolute_uri(),
                    })
                    subject = 'AcomeA memo app {}'.format(app_name)
                    from_email = 'memo@acomea.it'
                    to = email
                    text_content = plaintext.render(d)
                    html_content = htmly.render(d)
                    msg = EmailMultiAlternatives(subject, text_content, from_email, [to])
                    msg.attach_alternative(html_content, "text/html")
                    msg.send()
                    context.update({'email_message': u'Abbiamo inviato un memo alla email {0}'.format(email)})
                except ValidationError as e:
                    context.update({'err': u"L'email inserita non è valida"})
        context.update({'email_form': email_form_class})

        # LEAD PROFILE #
        if profilo.is_only_lead:
            # UTENTE LEAD: Versione Full - Intervallo di tempo
            angel_status = self.get_angel_status(profilo, instance.durata_demo)
            context.update({
                'label_button': instance.label_lead,
                'angel_status': angel_status,
                'user_status': 'loggato',
                'user_profile': 'lead',
            })

            if angel_status == 'attivo':
                shared_secret = acomea_full
                digest_sha = hashlib.sha1('%s:%s:%s' % (vendor_code, timestamp, shared_secret)).hexdigest()
                digest_base64 = base64.b64encode('%s:%s:%s' % (vendor_code, timestamp, digest_sha))
                context.update({'token': digest_base64})
                return context

            elif angel_status == 'non_attivo':
                shared_secret = acomea_limited
                digest_sha = hashlib.sha1('%s:%s:%s' % (vendor_code, timestamp, shared_secret)).hexdigest()
                digest_base64 = base64.b64encode('%s:%s:%s' % (vendor_code, timestamp, digest_sha))
                context.update({'token': digest_base64})
                return context

            else:
                if request.method == 'POST' and 'angel_active_submit' in request.POST:
                    angel_form = AngelForm(request.POST)
                    if angel_form.is_valid():
                        created, angel_attivazione = self.create_angel(profilo)
                        angel_status = self.get_angel_status(profilo, instance.durata_demo)
                        shared_secret = acomea_full
                        digest_sha = hashlib.sha1('%s:%s:%s' % (vendor_code, timestamp, shared_secret)).hexdigest()
                        digest_base64 = base64.b64encode('%s:%s:%s' % (vendor_code, timestamp, digest_sha))
                        context.update({'angel_status': angel_status, 'token': digest_base64})
                else:
                    angel_form = AngelForm()
                context.update({'angel_form': angel_form})

            return context

        # CLIENTE PROFILE #
        if profilo.is_cliente:
            # UTENTE CLIENTE: Versione Full
            angel_status = self.get_angel_status(profilo, instance.durata_demo)
            context.update({
                'label_button': instance.label_cliente,
                'angel_status': angel_status,
                'user_status': 'loggato',
                'user_profile': 'cliente',
            })

            if angel_status == 'attivo' or angel_status == 'non_attivo':
                shared_secret = acomea_full
                digest_sha = hashlib.sha1('%s:%s:%s' % (vendor_code, timestamp, shared_secret)).hexdigest()
                digest_base64 = base64.b64encode('%s:%s:%s' % (vendor_code, timestamp, digest_sha))
                context.update({'token': digest_base64})
                return context

            else:
                if request.method == 'POST' and 'angel_active_submit' in request.POST:
                    angel_form = AngelForm(request.POST)
                    if angel_form.is_valid():
                        created, angel_attivazione = self.create_angel(profilo)
                        angel_status = self.get_angel_status(profilo, instance.durata_demo)
                        shared_secret = acomea_full
                        digest_sha = hashlib.sha1('%s:%s:%s' % (vendor_code, timestamp, shared_secret)).hexdigest()
                        digest_base64 = base64.b64encode('%s:%s:%s' % (vendor_code, timestamp, digest_sha))
                        context.update({'angel_status': angel_status, 'token': digest_base64})
                else:
                    angel_form = AngelForm()
                context.update({'angel_form': angel_form})

            return context

        if profilo.is_acomea:
            # UTENTE ACOMEA: Versione Full
            angel_status = "attivo"
            context.update({
                'angel_status': angel_status,
                'user_status': 'loggato',
                'user_profile': 'acomea',
            })
            shared_secret = acomea_full
            digest_sha = hashlib.sha1('%s:%s:%s' % (vendor_code, timestamp, shared_secret)).hexdigest()
            digest_base64 = base64.b64encode('%s:%s:%s' % (vendor_code, timestamp, digest_sha))
            context.update({'token': digest_base64})
            return context





        return context

plugin_pool.register_plugin(AngelPlugin)